# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2018-12-12 16:36
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='TroodPluginModel',
            fields=[
                ('id', models.CharField(max_length=128, primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=128)),
                ('version', models.CharField(max_length=32)),
                ('active', models.BooleanField(default=False)),
                ('config', models.TextField()),
            ],
        ),
    ]
